@@ -12,6 +12,13 @@
 #pragma once
 
 #include <stdio.h>
+#include <limits.h>
+
+#ifndef PATH_MAX
+#define SIRI_PATH_MAX 4096
+#else
+#define SIRI_PATH_MAX PATH_MAX
+#endif
 
 int xpath_file_exist(const char * fn);
 int xpath_is_dir(const char * path);
