@@ -10,11 +10,13 @@ from test_select import TestSelect
 from test_series import TestSeries
 from test_server import TestServer
 from test_user import TestUser
+from test_compression import TestCompression
 
 Server.BUILDTYPE = 'Release'
 
 if __name__ == '__main__':
     # run_test(TestCluster())
+    run_test(TestCompression())
     run_test(TestGroup())
     run_test(TestList())
     run_test(TestInsert())
@@ -23,4 +25,3 @@ if __name__ == '__main__':
     run_test(TestSeries())
     run_test(TestServer())
     run_test(TestUser())
-
